@@ -11,7 +11,7 @@ BEGIN
 #       Needed to find perl dependencies shipped in deps tarball.
 if ($^O =~ /^aix/i) {
     unshift(@INC, qw(/usr/opt/perl5/lib/5.8.2/aix-thread-multi /usr/opt/perl5/lib/5.8.2 /usr/opt/perl5/lib/site_perl/5.8.2/aix-thread-multi /usr/opt/perl5/lib/site_perl/5.8.2));
-    }
+}
 use lib "$::XCATROOT/lib/perl";
 use strict;
 #-----------------------------------------------------------------------------
@@ -26,6 +26,7 @@ use strict;
   Example:
     my $retdata = xCAT::ServiceNodeUtils->readSNInfo;
 =cut
+
 #-----------------------------------------------------------------------------
 sub readSNInfo
 {
@@ -45,7 +46,7 @@ sub readSNInfo
             if (!($masternode))
             {
                 xCAT::MsgUtils->message('S',
-                                   "Could not get Master for node $nodename\n");
+                    "Could not get Master for node $nodename\n");
                 return 1;
             }
 
@@ -53,13 +54,13 @@ sub readSNInfo
             if ($et == 1)
             {
                 xCAT::MsgUtils->message('S',
-                                  "Could not get OS/ARCH for node $nodename\n");
+                    "Could not get OS/ARCH for node $nodename\n");
                 return 1;
             }
             if (!($et->{'os'} || $et->{'arch'}))
             {
                 xCAT::MsgUtils->message('S',
-                                  "Could not get OS/ARCH for node $nodename\n");
+                    "Could not get OS/ARCH for node $nodename\n");
                 return 1;
             }
         }
@@ -74,7 +75,7 @@ sub readSNInfo
 =head3 isServiceReq
 
 
-  Checks the service node table in the database to see 
+  Checks the service node table in the database to see
   if input Service should be setup on the
   input service node or Management Node (used by AAsn.pm)
 
@@ -83,7 +84,7 @@ sub readSNInfo
         hash of services to setup  for this service node
     Globals:
         $::RUNCMD_RC = 0; good
-        $::RUNCMD_RC = 1; error 
+        $::RUNCMD_RC = 1; error
     Error:
         none
     Example:
@@ -101,12 +102,12 @@ sub isServiceReq
     # read the  schema
     my $schema = xCAT::Table->getTableSchema("servicenode");
     my @services; #  list of only the actual service attributes from the servicenode table
-    my @servicesattrs;  # building second copy for call to getAllNodeAttribs, which modifies the array
-    foreach my $c (@{$schema->{cols}}) {
-      if (($c ne "node") && ($c ne "comments") && ($c ne "disable")) {
-       push @servicesattrs,$c;
-       push @services,$c;
-      }
+    my @servicesattrs; # building second copy for call to getAllNodeAttribs, which modifies the array
+    foreach my $c (@{ $schema->{cols} }) {
+        if (($c ne "node") && ($c ne "comments") && ($c ne "disable")) {
+            push @servicesattrs, $c;
+            push @services,      $c;
+        }
     }
 
     my @ips = @$serviceip;    # list of service node ip addresses and names
@@ -130,32 +131,32 @@ sub isServiceReq
         return;    # do not setup anything
     }
 
-    # Are we on the MN 
+    # Are we on the MN
     my $mname;
     if (xCAT::Utils->isMN()) {
-      my @nodeinfo = xCAT::NetworkUtils->determinehostname;
-       $mname   = pop @nodeinfo;                    # get hostname
+        my @nodeinfo = xCAT::NetworkUtils->determinehostname;
+        $mname = pop @nodeinfo;    # get hostname
     }
 
     my $servicehash;
-    
+
     # read all the nodes from the table, all the service attributes
-    my @snodelist= $servicenodetab->getAllNodeAttribs(\@servicesattrs); 
-    foreach my $service (@services) # check list of services
+    my @snodelist = $servicenodetab->getAllNodeAttribs(\@servicesattrs);
+    foreach my $service (@services)    # check list of services
     {
 
-        foreach $serviceip (@ips)    # check the table for this servicenode
+        foreach $serviceip (@ips)      # check the table for this servicenode
         {
             foreach my $node (@snodelist)
 
             {
                 if ($serviceip eq $node->{'node'})
-                {                    # match table entry
+                {                      # match table entry
                     if ($node->{$service})
-                    {                # returns service, only if set
+                    {                  # returns service, only if set
                         my $value = $node->{$service};
                         $value =~ tr/a-z/A-Z/;    # convert to upper
-                             # value 1 or yes  then we setup the service
+                            # value 1 or yes  then we setup the service
                         if (($value eq "1") || ($value eq "YES"))
                         {
                             $servicehash->{$service} = "1";
@@ -165,33 +166,35 @@ sub isServiceReq
                             $servicehash->{$service} = "0";
                         }
                     }
-                    last; 
+                    last;
                 }
-            }  
+            }
         }
 
     }
-    # if the ftpserver attribute is not defined in the service node table 
+
+    # if the ftpserver attribute is not defined in the service node table
     # and we are on
     # the Linux management node, we need to look at site.vsftp
     # if the tftpserver attribute is not defined, then we default it 1
     if (($mname) && (xCAT::Utils->isLinux())) {
-      if (!exists($servicehash->{'ftpserver'})) { 
-        my @tmp = xCAT::TableUtils->get_site_attribute("vsftp");
-        if ($tmp[0] && ($tmp[0] !~ /0|NO|No|no|N|n/ )) {
-           $servicehash->{'ftpserver'} = 1;
+        if (!exists($servicehash->{'ftpserver'})) {
+            my @tmp = xCAT::TableUtils->get_site_attribute("vsftp");
+            if ($tmp[0] && ($tmp[0] !~ /0|NO|No|no|N|n/)) {
+                $servicehash->{'ftpserver'} = 1;
+            }
+        }
+        if (!exists($servicehash->{'tftpserver'})) {
+            $servicehash->{'tftpserver'} = 1;
+        }
+
+        # On Ubuntu management node, we disabled the isc-dhcp-server in upstart,
+        # through file /etc/init/isc-dhcp-server.override, see bug 4399
+        # however, this causes a new problem, bug 4515
+        # the fix is to start dhcp server when starting xcatd
+        if (!exists($servicehash->{'dhcpserver'}) && xCAT::Utils->osver() =~ /ubuntu.*/i) {
+            $servicehash->{'dhcpserver'} = 1;
         }
-      }
-      if (!exists($servicehash->{'tftpserver'})) { 
-           $servicehash->{'tftpserver'} = 1;
-      }
-      # On Ubuntu management node, we disabled the isc-dhcp-server in upstart,
-      # through file /etc/init/isc-dhcp-server.override, see bug 4399
-      # however, this causes a new problem, bug 4515
-      # the fix is to start dhcp server when starting xcatd
-      if (!exists($servicehash->{'dhcpserver'}) && xCAT::Utils->osver() =~ /ubuntu.*/i) {
-          $servicehash->{'dhcpserver'} = 1;
-      }
     }
     $servicenodetab->close;
 
@@ -202,13 +205,13 @@ sub isServiceReq
 
 #-----------------------------------------------------------------------------
 
-=head3 getAllSN 
- 
-    Returns an array of all service nodes from service node table 
+=head3 getAllSN
+
+    Returns an array of all service nodes from service node table
 
     Arguments:
        ALL"  - will also return the management node in the array, if
-        if has been defined in the servicenode table 
+        if has been defined in the servicenode table
     Returns:
 		array of Service Nodes or empty array, if none
     Globals:
@@ -226,9 +229,10 @@ sub isServiceReq
 #-----------------------------------------------------------------------------
 sub getAllSN
 {
-   
+
     my ($class, $options) = @_;
     require xCAT::Table;
+
     # reads all nodes from the service node table
     my @servicenodes;
     my $servicenodetab = xCAT::Table->new('servicenode');
@@ -242,22 +246,24 @@ sub getAllSN
     my @nodes = $servicenodetab->getAllNodeAttribs(['tftpserver']);
     foreach my $nodes (@nodes)
     {
-          push @servicenodes, $nodes->{node};
+        push @servicenodes, $nodes->{node};
     }
+
     # if did not input "ALL" and there is a MN, remove it
     my @newservicenodes;
-    if ((!defined($options)) || ($options ne "ALL")) {   
-       my @mname = xCAT::Utils->noderangecontainsMn(@servicenodes);
-       if (@mname) { # if there is a MN
-        foreach my $node (@servicenodes) {
-          # check to see if node in MN list
-          if (!(grep(/^$node$/, @mname))) { # if node not in the MN array
-              push @newservicenodes, $node;
-          }
+    if ((!defined($options)) || ($options ne "ALL")) {
+        my @mname = xCAT::Utils->noderangecontainsMn(@servicenodes);
+        if (@mname) {    # if there is a MN
+            foreach my $node (@servicenodes) {
+
+                # check to see if node in MN list
+                if (!(grep(/^$node$/, @mname))) {  # if node not in the MN array
+                    push @newservicenodes, $node;
+                }
+            }
+            $servicenodetab->close;
+            return @newservicenodes;    # return without the MN in the array
         }
-        $servicenodetab->close;
-        return @newservicenodes;  # return without the MN in the array
-       }
     }
     $servicenodetab->close;
     return @servicenodes;
@@ -265,20 +271,20 @@ sub getAllSN
 
 #-----------------------------------------------------------------------------
 
-=head3 getSNandNodes 
- 
+=head3 getSNandNodes
+
     Returns an hash-array of all service nodes and the nodes they service
 
     Arguments:
-       none 
+       none
 #-----------------------------------------------------------------------------
 
-=head3 getSNandNodes 
- 
+=head3 getSNandNodes
+
     Returns an hash-array of all service nodes and the nodes they service
 
     Arguments:
-       none 
+       none
     Returns:
 	 Service Nodes and the nodes they service or empty , if none
     Globals:
@@ -297,6 +303,7 @@ sub getSNandNodes
 {
 
     require xCAT::Table;
+
     # read all the nodes from the nodelist table
     #  call get_ServiceNode to find which Service Node
     # the node belongs to.
@@ -315,24 +322,24 @@ sub getSNandNodes
 
 #-----------------------------------------------------------------------------
 
-=head3 getSNList 
- 
+=head3 getSNList
+
 	Reads the servicenode table. Will return all the enabled Service Nodes
 	that will setup the input Service ( e.g tftpserver,nameserver,etc)
 	If service is blank, then will return the list of all enabled Service
-	Nodes. 
+	Nodes.
 
     Arguments:
-       Servicename ( xcat,tftpserver,dhcpserver,conserver,etc) 
+       Servicename ( xcat,tftpserver,dhcpserver,conserver,etc)
        If no servicename, returns all Servicenodes
-       "ALL" argument means you also want the MN returned. It can be in the 
-       servicenode list.  If no "ALL", take out the MN if it is there. 
+       "ALL" argument means you also want the MN returned. It can be in the
+       servicenode list.  If no "ALL", take out the MN if it is there.
     Returns:
-	  Array of service node names 
+	  Array of service node names
     Globals:
         none
     Error:
-        1 - error  
+        1 - error
     Example:
          $sn= xCAT::ServiceNodeUtils->getSNList($servicename) { blah; }
          $sn= xCAT::ServiceNodeUtils->getSNList($servicename,"ALL") { blah; }
@@ -347,7 +354,38 @@ sub getSNandNodes
 sub getSNList
 {
     require xCAT::Table;
-    my ($class, $service,$options) = @_;
+    require xCAT::NodeRange;
+    my ($class, $service, $options) = @_;
+    my @servicenodes;
+    my $servicenodetab = xCAT::Table->new('servicenode');
+    my $nodetab = xCAT::Table->new('nodelist');
+    unless ($servicenodetab)    # no  servicenode table
+    {
+        xCAT::MsgUtils->message('I', "Unable to open servicenode table.\n");
+        $servicenodetab->close;
+        return @servicenodes;
+    }
+
+
+    if($service){
+        @servicenodes = $servicenodetab->getAllAttribsWhere(["$service==1"],'node');
+    }else{
+        @servicenodes = $servicenodetab->getAllAttribs(('node'));
+    }
+
+    @servicenodes=xCAT::NodeRange::noderange(join(',',map {$_->{node}} @servicenodes));
+    if($options eq "ALL"){
+        return @servicenodes;
+    }
+
+    @servicenodes = $nodetab->getAllAttribsWhere("node in ("."\'".join("\',\'", @servicenodes)."\'".") and groups not like '%__mgmtnode%'",'node');
+    return map {$_->{node}} @servicenodes;
+}
+
+sub getSNList_orig
+{
+    require xCAT::Table;
+    my ($class, $service, $options) = @_;
 
     # reads all nodes from the service node table
     my @servicenodes;
@@ -359,19 +397,20 @@ sub getSNList
     }
     my @nodes = $servicenodetab->getAllNodeAttribs([$service]);
     $servicenodetab->close;
+
     foreach my $node (@nodes)
     {
-        if  (! defined ($service) || ($service eq ""))     # want all the service nodes
+        if (!defined($service) || ($service eq "")) # want all the service nodes
         {
             push @servicenodes, $node->{node};
         }
         else
-        {                       # looking for a particular service
+        {    # looking for a particular service
             if ($node->{$service})
-            {                   # if null then do not add node
+            {    # if null then do not add node
                 my $value = $node->{$service};
                 $value =~ tr/a-z/A-Z/;    # convert to upper
-                     # value 1 or yes or blank then we setup the service
+                    # value 1 or yes or blank then we setup the service
                 if (($value == 1) || ($value eq "YES"))
                 {
                     push @servicenodes, $node->{node};
@@ -380,18 +419,19 @@ sub getSNList
             }
         }
     }
+
     # if did not input "ALL" and there is a MN, remove it
     my @newservicenodes;
-    if ((!defined($options)) || ($options ne "ALL")) {   
-       my $mname = xCAT::Utils->noderangecontainsMn(@servicenodes);
-       if ($mname) { # if there is a MN
-        foreach my $nodes (@servicenodes) {
-           if ($mname ne ($nodes)){
-              push @newservicenodes, $nodes;
-           }
+    if ((!defined($options)) || ($options ne "ALL")) {
+        my $mname = xCAT::Utils->noderangecontainsMn(@servicenodes);
+        if ($mname) {    # if there is a MN
+            foreach my $nodes (@servicenodes) {
+                if ($mname ne ($nodes)) {
+                    push @newservicenodes, $nodes;
+                }
+            }
+            return @newservicenodes;    # return without the MN in the array
         }
-        return @newservicenodes;  # return without the MN in the array
-       }
     }
 
     return @servicenodes;
@@ -403,7 +443,7 @@ sub getSNList
 =head3 get_ServiceNode
 
      Will get the Service node ( name or ipaddress) as known by the Management
-	 Node  or Node for the input nodename or ipadress of the node 
+	 Node  or Node for the input nodename or ipadress of the node
          which can be a Service Node.
          If the input node is a Service Node then it's Service node
          is always the Management Node.
@@ -466,21 +506,25 @@ sub get_ServiceNode
     {
         $snattribute = "xcatmaster";
     }
-    # get site.master this will be the default
-    my $master = xCAT::TableUtils->get_site_Master();  
-    $noderestab = xCAT::Table->new('noderes');
 
+    # get site.master this will be the default
+    my $master = xCAT::TableUtils->get_site_Master();
+    unless($master){
+        xCAT::MsgUtils->message('SW',"site.master is not set!\n");
+    }
+
+    $noderestab = xCAT::Table->new('noderes');
     unless ($noderestab)    # no noderes table, use default site.master
     {
         xCAT::MsgUtils->message('I',
-                         "Unable to open noderes table. Using site->Master.\n");
+            "Unable to open noderes table. Using site->Master.\n");
 
         if ($master)        # use site Master value
         {
-				
+
             foreach my $node (@node_list)
-            {               
-					push @{$snhash{$master}}, $node;
+            {
+                push @{ $snhash{$master} }, $node;
             }
         }
         else
@@ -500,16 +544,16 @@ sub get_ServiceNode
 
         foreach my $node (@node_list)
         {
-            foreach my $rec (@{$nodehash->{$node}})
+            foreach my $rec (@{ $nodehash->{$node} })
             {
-                if ($rec and $rec->{$snattribute}) # use noderes.servicenode
+                if ($rec and $rec->{$snattribute})    # use noderes.servicenode
                 {
                     my $key = $rec->{$snattribute};
-                    push @{$snhash{$key}}, $node;
+                    push @{ $snhash{$key} }, $node;
                 }
-                else  # use site.master
-                {    
-  		  push @{$snhash{$master}}, $node;
+                elsif($master)                                  # use site.master
+                {
+                    push @{ $snhash{$master} }, $node;
                 }
             }
         }
@@ -527,10 +571,10 @@ sub get_ServiceNode
         {
             $nodehash =
               $noderestab->getNodesAttribs(\@node_list,
-                                           [$service, $snattribute]);
+                [ $service, $snattribute ]);
             foreach my $node (@node_list)
             {
-                foreach my $rec (@{$nodehash->{$node}})
+                foreach my $rec (@{ $nodehash->{$node} })
                 {
                     if ($rec and $rec->{$service})
                     {
@@ -545,18 +589,18 @@ sub get_ServiceNode
                                 $key = $nodeattr;
                             }
                         }
-                        push @{$snhash{$key}}, $node;
+                        push @{ $snhash{$key} }, $node;
                     }
                     else
                     {
                         if ($rec and $rec->{$snattribute})    # if it exists
                         {
                             my $key = $rec->{$snattribute};
-                            push @{$snhash{$key}}, $node;
+                            push @{ $snhash{$key} }, $node;
                         }
-                        else
+                        elsif($master)
                         {                                     # use site.master
-                            push @{$snhash{$master}}, $node;
+                            push @{ $snhash{$master} }, $node;
                         }
                     }
                 }
@@ -576,23 +620,23 @@ sub get_ServiceNode
                 unless ($nodehmtab)    # no nodehm table
                 {
                     xCAT::MsgUtils->message('I',
-                                            "Unable to open nodehm table.\n");
+                        "Unable to open nodehm table.\n");
 
                     # use servicenode
                     $nodehash =
                       $noderestab->getNodesAttribs(\@node_list, [$snattribute]);
                     foreach my $node (@node_list)
                     {
-                        foreach my $rec (@{$nodehash->{$node}})
+                        foreach my $rec (@{ $nodehash->{$node} })
                         {
                             if ($rec and $rec->{$snattribute})
                             {
                                 my $key = $rec->{$snattribute};
-                                push @{$snhash{$key}}, $node;
+                                push @{ $snhash{$key} }, $node;
                             }
-                            else
+                            elsif($master)
                             {    # use site.master
-                                push @{$snhash{$master}}, $node;
+                                push @{ $snhash{$master} }, $node;
                             }
                         }
                     }
@@ -605,7 +649,7 @@ sub get_ServiceNode
                   $nodehmtab->getNodesAttribs(\@node_list, ['conserver']);
                 foreach my $node (@node_list)
                 {
-                    foreach my $rec (@{$nodehash->{$node}})
+                    foreach my $rec (@{ $nodehash->{$node} })
                     {
                         if ($rec and $rec->{'conserver'})
                         {
@@ -622,21 +666,21 @@ sub get_ServiceNode
                                     $key = $nodeattr;
                                 }
                             }
-                            push @{$snhash{$key}}, $node;
+                            push @{ $snhash{$key} }, $node;
                         }
                         else
                         {              # use service node for this node
                             $sn =
                               $noderestab->getNodeAttribs($node,
-                                                          [$snattribute]);
+                                [$snattribute]);
                             if ($sn and $sn->{$snattribute})
                             {
                                 my $key = $sn->{$snattribute};
-                                push @{$snhash{$key}}, $node;
+                                push @{ $snhash{$key} }, $node;
                             }
-                            else
+                            elsif($master)
                             {          # no service node use master
-                                push @{$snhash{$master}}, $node;
+                                push @{ $snhash{$master} }, $node;
                             }
                         }
                     }
@@ -649,7 +693,7 @@ sub get_ServiceNode
             else
             {
                 xCAT::MsgUtils->message('E',
-                                        "Invalid service=$service input.\n");
+                    "Invalid service=$service input.\n");
                 $::ERROR_RC = 1;
             }
         }
@@ -667,7 +711,7 @@ sub get_ServiceNode
 	 as known by the Management
 	 Server or Node for the input nodename or ipadress of the node
 	 It will then format the output into a single servicenode key with values
-	 the list of nodes service by that service node.  This routine will 
+	 the list of nodes service by that service node.  This routine will
 	 break up pools of service nodes into individual node in the hash unlike
 	 get_ServiceNode which leaves the pool as the key.
 
@@ -685,7 +729,7 @@ sub get_ServiceNode
 		specific service like tftp
 
 
-	 output: A hash ref  of arrays, the key is a single service node 
+	 output: A hash ref  of arrays, the key is a single service node
 	          pointing to
 			 a list of nodes that are serviced by that service node
 	        'rra000-m'=>['blade01', 'testnode']
@@ -711,47 +755,47 @@ sub getSNformattedhash
     my $cmd;
     my %newsnhash;
 
-	my $type="";
-	if ($btype) {
-		$type=$btype;
-	}
+    my $type = "";
+    if ($btype) {
+        $type = $btype;
+    }
 
-	# get the values of either the servicenode or xcatmaster attributes
+    # get the values of either the servicenode or xcatmaster attributes
     my $sn = xCAT::ServiceNodeUtils->get_ServiceNode(\@node_list, $service, $request);
 
     # get the keys which are the service nodes and break apart any pool lists
     # format into individual service node keys pointing to node lists
-	if ($sn)
-	{
+    if ($sn)
+    {
         foreach my $snkey (keys %$sn)
         {
-			# split the key if pool of service nodes
-			push my @tmpnodes, $sn->{$snkey};
-			my @nodes;
-			for my $i (0 .. $#tmpnodes) {
-				for my $j ( 0 .. $#{$tmpnodes[$i]}) {
-					my $check=$tmpnodes[$i][$j];
-					push @nodes,$check; 
-				}
-			}
+            # split the key if pool of service nodes
+            push my @tmpnodes, $sn->{$snkey};
+            my @nodes;
+            for my $i (0 .. $#tmpnodes) {
+                for my $j (0 .. $#{ $tmpnodes[$i] }) {
+                    my $check = $tmpnodes[$i][$j];
+                    push @nodes, $check;
+                }
+            }
 
-			# for SN backup we might only want the primary or backup
-			my @servicenodes;
-			my ($primary, $backup) = split /,/, $snkey;
-			if (($primary) && ($type eq "primary")) {
-				push @servicenodes, $primary;
-			} elsif (($backup) && ($type eq "backup")) {
-				push @servicenodes, $backup;
-			} else {
-				@servicenodes = split /,/, $snkey;
-			}
+            # for SN backup we might only want the primary or backup
+            my @servicenodes;
+            my ($primary, $backup) = split /,/, $snkey;
+            if (($primary) && ($type eq "primary")) {
+                push @servicenodes, $primary;
+            } elsif (($backup) && ($type eq "backup")) {
+                push @servicenodes, $backup;
+            } else {
+                @servicenodes = split /,/, $snkey;
+            }
 
-			# now build new hash of individual service nodes
-			foreach my $newsnkey (@servicenodes) {
-				push @{$newsnhash{$newsnkey}}, @nodes;
-			}
-		}
-	}
+            # now build new hash of individual service nodes
+            foreach my $newsnkey (@servicenodes) {
+                push @{ $newsnhash{$newsnkey} }, @nodes;
+            }
+        }
+    }
     return \%newsnhash;
 }
 
@@ -792,39 +836,39 @@ sub getAIXSNinterfaces
         my $out = xCAT::InstUtils->xcmd($callback, $sub_req, "xdsh", $sn, $ifcmd, 0);
         if ($::RUNCMD_RC != 0)
         {
-			my $rsp;
-			push @{$rsp->{data}}, "Could not get IP addresses from service node $sn.\n";
-			xCAT::MsgUtils->message("E", $rsp, $callback);
+            my $rsp;
+            push @{ $rsp->{data} }, "Could not get IP addresses from service node $sn.\n";
+            xCAT::MsgUtils->message("E", $rsp, $callback);
             next;
         }
 
-		my @result;
-		foreach my $line ( split(/\n/, $out)) {
-			$line =~ s/$sn:\s+//;
-			push(@result, $line);
-		}
-
-		foreach my $int (@result) {
-			my ($inet, $SNIP, $str) = split(" ", $int);
-			chomp $SNIP;
-			$SNIP =~ s/\/.*//; # ipv6 address 4000::99/64
-			$SNIP =~ s/\%.*//; # ipv6 address ::1%1/128
-            push(@{$SNinterfaces{$sn}}, $SNIP);
+        my @result;
+        foreach my $line (split(/\n/, $out)) {
+            $line =~ s/$sn:\s+//;
+            push(@result, $line);
         }
-    } # end foreach SN
-   	return \%SNinterfaces;
+
+        foreach my $int (@result) {
+            my ($inet, $SNIP, $str) = split(" ", $int);
+            chomp $SNIP;
+            $SNIP =~ s/\/.*//;    # ipv6 address 4000::99/64
+            $SNIP =~ s/\%.*//;    # ipv6 address ::1%1/128
+            push(@{ $SNinterfaces{$sn} }, $SNIP);
+        }
+    }    # end foreach SN
+    return \%SNinterfaces;
 }
 
 #-----------------------------------------------------------------------------
 
-=head3  
- 
-    getSNandCPnodes - Take an array of nodes and returns 
-    an array of the service 
-    nodes and an array of the computenode . 
+=head3
+
+    getSNandCPnodes - Take an array of nodes and returns
+    an array of the service
+    nodes and an array of the computenode .
 
     Arguments:
-       none 
+       none
     Returns:
 		array of Service Nodes and/or array of compute nodesarray of compute nodes       empty array, if none
     Globals:
@@ -839,21 +883,22 @@ sub getAIXSNinterfaces
 =cut
 
 #-----------------------------------------------------------------------------
-sub getSNandCPnodes 
+sub getSNandCPnodes
 {
-   
-    my ($class, $nodes,$sn,$cn) = @_; 
+
+    my ($class, $nodes, $sn, $cn) = @_;
     my @nodelist = @$nodes;
+
     # get the list of all Service nodes
-    my @allSN=xCAT::ServiceNodeUtils->getAllSN;
+    my @allSN = xCAT::ServiceNodeUtils->getAllSN;
     foreach my $node (@nodelist) {
-      if (grep(/^$node$/, @allSN)) { # it is a SN
-         push (@$sn,$node);
-      } else {  # a CN
-         push (@$cn,$node);
-      }
+        if (grep(/^$node$/, @allSN)) {    # it is a SN
+            push(@$sn, $node);
+        } else {                          # a CN
+            push(@$cn, $node);
+        }
     }
 
-    return ; 
+    return;
 }
 1;
